from .reviewer import *

__copyright__    = 'Copyright (C) 2020 Yohei Kirino'
__version__      = '0.1.1'
__license__      = 'MIT'
__author__       = 'Yohei Kirino'
__author_email__ = 'kirino.yohei@gmail.com'
__url__          = 'https://github.com/Gappo-Gappo/ikku'

__all__ = ['Reviewer']